# This R script requires the Coda and lattice packages to be installed
# or will fail.

# Distributed as part of PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
# version 0.94 - beta version July, 2012
# Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
# jason.de.koning@gmail.com http://jasondk.org
# wanjungu@gmail.com
# David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com

# This program was created and distributed by the
# David Pollock laboratory, University of Colorado School of Medicine
# copyright © 2012 David Pollock
# All rights reserved.
# http://www.evolutionarygenomics.com

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

library("lattice");
library("coda");

# Assume the PLEX matrix output is in matrixoutfile
t <- read.table("./matrixoutfile", header=TRUE );

# Use CODA to calculate ESS's
ESS <- effectiveSize( t[,5:length(t[1,]) ] );

# Remove the ESS values for the diagonal of the rate matrix
N <- sqrt( (length(t[1,]) - 4 ) );
for (i in 1:N) {
	for (j in 1:N) {
		if (j == i) {
			ESS[ (((i-1)*N)+(j-1)+1) ] = NA;
		}
	}
}

# Output the mean +/- SE rate parameter ESSs, as well as the mean +/- SE ESS as a % of number of samples
cat( mean(ESS, na.rm=TRUE), "\t", sqrt(var(ESS, na.rm=TRUE)), "\t", ( mean(ESS, na.rm=TRUE) / length(t[,1]) ), "\t", sqrt( var(ESS/length(t[,1]), na.rm=TRUE)), "\n");
