# Plots the log-likelihood for three runs, with burnin as last parameter...


# Distributed as part of PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
# version 0.94 - beta version July, 2012
# Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
# jason.de.koning@gmail.com http://jasondk.org
# wanjungu@gmail.com
# David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com

# This program was created and distributed by the
# David Pollock laboratory, University of Colorado School of Medicine
# copyright © 2012 David Pollock
# All rights reserved.
# http://www.evolutionarygenomics.com

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.


args <- commandArgs();
numFiles <- length(args) - 5;
print(c("Numfiles is ", numFiles));

theColors <- rainbow(n=10);
#rgb(0,0,0); 
#colors();

burnin <- as.numeric(args[5]);

data <-  vector(mode="list", length=numFiles);

for (i in 1:numFiles) {
	theData <- read.table(args[(5+i)], sep='\t', header=T);
	data[[i]] <- data.frame(theData);
}


maxY <- -9999999999999;
maxX <- 0;
minY <- 9999999999;

for (i in 1:numFiles) {
	maxY <- max(maxY, data[[i]]$LnLikelihood[data[[i]]$Count > burnin]);
	minY <- min(minY, data[[i]]$LnLikelihood[data[[i]]$Count > burnin]);
	maxX <- max(maxX, data[[i]]$Count);
}	

pdf(file="./plot.pdf", bg="transparent", version="1.4");
for (i in 1:numFiles) {
	if (i == 1) {
		plot( data[[i]]$Count[data[[i]]$Count > burnin], data[[i]]$LnLikelihood[data[[i]]$Count > burnin], xlim=c( as.numeric(burnin), maxX), ylim=c(minY, maxY), col=theColors[i], type='l', xlab="Generation", ylab="log(likelihood)");
	} else {
		lines( data[[i]]$Count[data[[i]]$Count > burnin], data[[i]]$LnLikelihood[data[[i]]$Count > burnin], col=theColors[i]);
	}
#	theMean <- mean(data[[i]]$LnLikelihood[data[[i]]$Count > burnin]);
#	abline(h=theMean, col=theColors[i], lwd=2);
}
dev.off();
