/*
 *  AbstractMatrix.h
 *  Originally created by Chip Jarred for David Pollock's CLAMP/PLEX program
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

 /*
 This class is the abstract class of matrix,
 include matrix construction, deconstruction, fill, resize,
 get the rows, cols, swap the elements, 
 test the empty of matrix, and operator = and []
 commented by Wanjun
 */

#ifndef AbstractMatrix_h__
#define AbstractMatrix_h__

#include <algorithm>
#include "Vector.h"

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
class AbstractMatrix
{
	public:
				typedef	Vector<DataType>		RowType;
				typedef Vector<RowType>			RowVector;
				typedef	unsigned int			size_type;
				
												AbstractMatrix(void);
												AbstractMatrix(
													size_type			inRows,
													size_type			inColumns,
													const DataType&		inValue = DataType());
												AbstractMatrix(
													const AbstractMatrix&		inAbstractMatrix);
		virtual									~AbstractMatrix() {}
													
		virtual	void							fill(
													const DataType&		inVal) = 0;
				
		virtual	void							resize(
													size_type			inRows,
													size_type			inColumns,
													const DataType&		inVal = DataType()) = 0;
													
		virtual void							clear(void);

				bool							empty(void) const;
													
				size_type						rows(void) const;											
				size_type						cols(void) const;
				
		virtual	const RowType&					operator[](
													size_type			inRowIndex) const = 0;
		virtual	RowType&						operator[](
													size_type			inRowIndex) = 0;
													
//				const AbstractMatrix&			operator=(
//													const AbstractMatrix&	inAbstractMatrix);
//				const AbstractMatrix&			operator=(
//													DataType**				inAbstractMatrix);
													
		virtual	void							SwapRows(
													size_type			inRow1,
													size_type			inRow2) = 0;
		virtual	void							SwapColumns(
													size_type			inColumn1,
													size_type			inColumn2) = 0;
													
				void							swap(
													AbstractMatrix&				inAbstractMatrix);
															
	private:
				size_type						mNumRows;
				size_type						mNumColumns;
};

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
AbstractMatrix<DataType>::AbstractMatrix(void)
	:	mNumRows(0),
		mNumColumns(0)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
AbstractMatrix<DataType>::AbstractMatrix(
	size_type		inRows,
	size_type		inColumns,
	const DataType&	inValue)
	:	mNumRows(0),
		mNumColumns(0)
{
	resize(inRows, inColumns, inValue);
}

//---------------------------------------------------------------------------
template <typename DataType>
AbstractMatrix<DataType>::AbstractMatrix(
	const AbstractMatrix&	inAbstractMatrix)
	:	mNumRows(inAbstractMatrix.mNumRows),
		mNumColumns(inAbstractMatrix.mNumColumns)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
void
AbstractMatrix<DataType>::resize(
	size_type		inRows,
	size_type		inColumns,
	const DataType&	inVal)
{
	mNumRows	= inRows;
	mNumColumns = inColumns;
}

//---------------------------------------------------------------------------
template <typename DataType>
void
AbstractMatrix<DataType>::clear(void)
{
	mNumRows = 0;
	mNumColumns = 0;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
bool
AbstractMatrix<DataType>::empty(void) const
{
	return mNumRows == 0 || (mNumColumns == 0);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
typename AbstractMatrix<DataType>::size_type
AbstractMatrix<DataType>::rows(void) const
{
	return mNumRows;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
typename AbstractMatrix<DataType>::size_type
AbstractMatrix<DataType>::cols(void) const
{
	return mNumColumns;
}

//---------------------------------------------------------------------------
/*template <typename DataType>
inline
const AbstractMatrix<DataType>&
AbstractMatrix<DataType>::operator=(
	const AbstractMatrix&	inAbstractMatrix)
{
	AbstractMatrix	temp = inAbstractMatrix;
	
	swap(temp);
	return *this;
}
*/
//---------------------------------------------------------------------------
template <typename DataType>
inline
void
AbstractMatrix<DataType>::swap(
	AbstractMatrix&	inAbstractMatrix)
{
	std::swap(mNumRows, inAbstractMatrix.mNumRows);
	std::swap(mNumColumns, inAbstractMatrix.mNumColumns);
}

//---------------------------------------------------------------------------
#endif // AbstractMatrix_h__
