/*
 *  AbstractMatrix3D.h
 *  Originally created by Chip Jarred for David Pollock's CLAMP/PLEX program
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

 /*
 This class is the abstract class of 3 dimensional matrix,
 include 3d matrix construction, deconstruction, fill, resize,
 get the rows, cols, and layers, swap the elements, 
 test the empty of matrix, and operator = and []
 commented by Wanjun
 */

#ifndef AbstractMatrix3D_h__
#define AbstractMatrix3D_h__

#include "Matrix.h"

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
class AbstractMatrix3D
{
	public:
			typedef	Matrix<DataType>		RowType;
			typedef Vector<RowType>			RowVector;
			typedef	unsigned int			size_type;
			
											AbstractMatrix3D(void);
											AbstractMatrix3D(
												size_type			inRows,
												size_type			inColumns,
												size_type			inLayers,
												const DataType&		inValue = DataType());
											AbstractMatrix3D(
												const AbstractMatrix3D&		inMatrix);
	virtual									~AbstractMatrix3D(void) {}
												
	virtual	void							fill(
												const DataType&		inVal) = 0;
			
	virtual	void							resize(
												size_type			inRows,
												size_type			inColumns,
												size_type			inLayers,
												const DataType&		inVal = DataType()) = 0;
												
			bool							empty(void) const;
												
			size_type						rows(void) const;											
			size_type						cols(void) const;
			size_type						layers(void) const;
			
	virtual	const RowType&					operator[](
												size_type			inRowIndex) const = 0;
	virtual	RowType&						operator[](
												size_type			inRowIndex) = 0;
												
//			const AbstractMatrix3D&			operator=(
//												const AbstractMatrix3D&		inMatrix);
//			const AbstractMatrix3D&			operator=(
//												DataType***			inMatrix);
												
	virtual	void							SwapRows(
												size_type			inRow1,
												size_type			inRow2) = 0;
	virtual	void							SwapColumns(
												size_type			inColumn1,
												size_type			inColumn2) = 0;
	virtual	void							SwapLayers(
												size_type			inLayer1,
												size_type			inLayer2)  = 0;
												
			void							swap(
												AbstractMatrix3D&			inMatrix);
															
	private:
			size_type						mNumRows;
			size_type						mNumColumns;
			size_type						mNumLayers;
};

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
AbstractMatrix3D<DataType>::AbstractMatrix3D(void)
	:	mNumRows(0),
		mNumColumns(0),
		mNumLayers(0)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
AbstractMatrix3D<DataType>::AbstractMatrix3D(
	size_type		inRows,
	size_type		inColumns,
	size_type		inLayers,
	const DataType&	inValue)
	:	mNumRows(inRows),
		mNumColumns(inColumns),
		mNumLayers(inLayers)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
AbstractMatrix3D<DataType>::AbstractMatrix3D(
	const AbstractMatrix3D&	inMatrix)
	:	mNumRows(inMatrix.mNumRows),
		mNumColumns(inMatrix.mNumColumns),
		mNumLayers(inMatrix.mNumLayers)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
void
AbstractMatrix3D<DataType>::resize(
	size_type		inRows,
	size_type		inColumns,
	size_type		inLayers,
	const DataType&	inVal)
{
	mNumRows	= inRows;
	mNumColumns = inColumns;
	mNumLayers	= inLayers;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
bool
AbstractMatrix3D<DataType>::empty(void) const
{
	return (mNumRows == 0) || (mNumColumns == 0) || (mNumLayers == 0);
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
typename AbstractMatrix3D<DataType>::size_type
AbstractMatrix3D<DataType>::rows(void) const
{
	return mNumRows;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
typename AbstractMatrix3D<DataType>::size_type
AbstractMatrix3D<DataType>::cols(void) const
{
	return mNumColumns;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
typename AbstractMatrix3D<DataType>::size_type
AbstractMatrix3D<DataType>::layers(void) const
{
	return mNumLayers;
}

//---------------------------------------------------------------------------
/*template <typename DataType>
inline
const AbstractMatrix3D<DataType>&
AbstractMatrix3D<DataType>::operator=(
	const AbstractMatrix3D&	inMatrix)
{
	AbstractMatrix3D	temp = inMatrix;
	
	swap(temp);
	return *this;
} */

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
AbstractMatrix3D<DataType>::swap(
	AbstractMatrix3D&	inMatrix)
{
	std::swap(mNumRows, inMatrix.mNumRows);
	std::swap(mNumColumns, inMatrix.mNumColumns);
	std::swap(mNumLayers, inMatrix.mNumLayers);
}

//---------------------------------------------------------------------------
#endif // AbstractMatrix3D_h__
