/*
 *  AbstractVector.h
 *  Originally created by Chip Jarred for David Pollock's CLAMP/PLEX program
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

 /*
 This class is the abstract class of vector,
 include vector construction, deconstruction, fill, resize,
 get the rows, cols, and layers, swap the elements, 
 test the empty of vector, and operator = and []
 commented by Wanjun
 */

#ifndef AbstractVector_h__
#define AbstractVector_h__

#include <algorithm>
#include <cassert>

//---------------------------------------------------------------------------
template <typename DataType>
class AbstractVector
{
	public:
			typedef	unsigned int		size_type;
			
										AbstractVector(void);
										AbstractVector(
											size_type				inNumItems,
											const DataType&			inVal = DataType());
										AbstractVector(
											const AbstractVector&	inAbstractVector);
	virtual								~AbstractVector(void);
										
//			const AbstractVector&		operator=(
//											const AbstractVector&	inAbstractVector);
											
	virtual	bool						empty(void)		const;
	virtual	size_type					size(void)		const;
	virtual	void						resize(
											size_type		inNewSize,
											const DataType&	inVal = DataType()) = 0;
	virtual	void						fill(
											const DataType&	inVal) = 0;
											
	virtual	DataType&					operator[](
											size_type		inIndex) = 0;
	virtual	const DataType&				operator[](
											size_type		inIndex) const = 0;
											
			void						swap(
											AbstractVector&			ioAbstractVector);
	private:
			size_type					mNumItems;
};

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType>
AbstractVector<DataType>::AbstractVector(void)
	:	mNumItems(0)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
AbstractVector<DataType>::AbstractVector(
	size_type		inNumItems,
	const DataType&	inVal)
	:	mNumItems(inNumItems)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
AbstractVector<DataType>::AbstractVector(
	const AbstractVector&	inAbstractVector)
	:	mNumItems(inAbstractVector.mNumItems)
{
}

//---------------------------------------------------------------------------
template <typename DataType>
AbstractVector<DataType>::~AbstractVector(void)
{
}

//---------------------------------------------------------------------------
/*template <typename DataType>
inline
const AbstractVector<DataType>&
AbstractVector<DataType>::operator=(
	const AbstractVector&	inAbstractVector)
{
	AbstractVector	temp(inAbstractVector);
	swap(temp);
	return *this;
}
*/

//---------------------------------------------------------------------------
template <typename DataType>
inline
bool
AbstractVector<DataType>::empty(void) const
{
	return mNumItems == 0;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
typename AbstractVector<DataType>::size_type
AbstractVector<DataType>::size(void) const
{
	return mNumItems;
}

//---------------------------------------------------------------------------
template <typename DataType>
void
AbstractVector<DataType>::resize(
	size_type		inNewSize,
	const DataType&	inVal)
{
	if (inNewSize != mNumItems)
		mNumItems = inNewSize;
}

//---------------------------------------------------------------------------
template <typename DataType>
inline
void
AbstractVector<DataType>::swap(
	AbstractVector&	inAbstractVector)
{
	std::swap(inAbstractVector.mNumItems, mNumItems);
}



#endif // AbstractVector_h__
