/*
 *  GTRModel.h
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef GTRModel_h__
#define GTRModel_h__


#include "MixtureModel.h"

//---------------------------------------------------------------------------
class GTRModel : public MixtureModel
{
	public:
		GTRModel(
			const DoubleMatrix3D& inAlphaMatrix3D = DoubleMatrix3D(),
			const DoubleMatrix& inFrequencyMatrix = DoubleMatrix(),
			const IntMatrix& inCountMatrix = IntMatrix());

		GTRModel(
			const GTRModel&	inModel);
			
		virtual	~GTRModel(
			void);
									
	//	virtual const GTRModel& operator=(		//copy a GTR model
	//		const GTRModel&	inModel);
					
		virtual void swap(				//swap a GTR model
			GTRModel& inModel);
			
		virtual void PrintAlphaMatrix3D(		//print the alpha matrix for each model to outStream
			ostream& outStream, ObservedData& theData);
			
		void PrintCountMatrix(			//print the count matrix to outStream
			ostream& outStream, ObservedData& theData);

		virtual void PrintFrequencyMatrix(		//print the state frequency vector of each model to outStream
			ostream& outStream, ObservedData& theData);
    
        virtual void PrintStationaryFrequencyMatrix(		//print the state frequency vector of each model to outStream
                                      ostream& outStream, ObservedData& theData);

		virtual void PrintRateMatrix3D(
				ostream&		outStream, ObservedData& theData);
			
		virtual void InitializeParameter(	//initialize the parameters of GTR model based on state changes count matrix
			const IntMatrix& inCountMatrix);
			
		virtual void UpdateParameter(		//update all the parameters for each model in mixture model,
			const int& m_nMethod);		//given the conditional pathway calculation method
		
	virtual void GTRModelParameterstoRates(		//transform the alpha matrix and frequency parameters for each model to rate matrix
										   void);
	
	
	//		DoubleMatrix& returnStateFrequencies(void) { return mFrequencyMatrix; }

	protected:
		
	private:
		virtual void InitializeFrequencyMatrix(		//initialize the frequency vector for each model
			const IntMatrix& inCountMatrix);
			
		virtual void InitializeAlphaMatrix3D(		//initialize the rate matrix for each model
			const IntMatrix& inCountMatrix);	
	
		virtual void UpdateFrequency(			//update the frequency vector for a single model
			DoubleVector& mFrequency);

		virtual void UpdateFrequencyMatrix(		//update the frequency vectors for all models
			void);
			
		virtual void UpdateAlphaMatrix(			//update the alpha matrix for a single model
			DoubleMatrix& mAlphaMatrix, 
			const DoubleVector& mFrequency, 
			const int& m_nMethod);
			
	void UpdateAlphaMatrix(int theModel, const int& m_nMethod);

		virtual void UpdateAlphaMatrix3D(		//update the alpha matrix for all models
			const int& m_nMethod);
			
		virtual void SetCountMatrix(			//set the count matrix for this model
			const IntMatrix& inCountMatrix);

		virtual void UpdateFrequencyParameter(		//update a single frequency parameter
			double& inParameter);

		virtual void UpdateAlphaParameter(		//update a single alpha parameter
			double& inParameter);
		
		int UpdateAlphaParameter( int theModel, int i, int j, int reversibleEvaluation);
	
//		DoubleMatrix	mFrequencyMatrix;	//frequency vector for each model
		IntMatrix	mCountMatrix;		//count matrix of state changes in alignment
};

#endif // GTRModel_h__
