/*
 *  KeyedRecyclery.h
 *  Originally created by Chip Jarred for David Pollock's CLAMP/PLEX program
 *  Copyright (c) 2004-2005 Dr. David Pollock. All rights reserved.
 
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef KeyedRecyclery_h__
#define KeyedRecyclery_h__

#include "Recyclery.h"

#include <map>

template <typename DataType, typename Initializer>
class KeyedRecyclery
{
	public:
													KeyedRecyclery(void);
													KeyedRecyclery(
														const Initializer&	inInitializer);
													KeyedRecyclery(
														const KeyedRecyclery&	inSrc);
													~KeyedRecyclery(void);
													
				const KeyedRecyclery&				operator=(
															const KeyedRecyclery& inSrc);

				DataType*							New(
														const Initializer&	inInitializer);
				void								Dispose(
														const Initializer&	inInitializer,
														DataType*&	ioPtr);
									
				
	protected:
	private:
				void								CreateNewRecycleryIfNeeded(
														const Initializer&	inInitializer);

		typedef std::map<Initializer, Recyclery<DataType, Initializer> >	RecycleryMap;
				RecycleryMap						mRecycleries;
};

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
KeyedRecyclery<DataType, Initializer>::KeyedRecyclery(void)
	:	mRecycleries()
{
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
KeyedRecyclery<DataType, Initializer>::KeyedRecyclery(
	const KeyedRecyclery&	inSrc)
	:	mRecycleries()
{
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
KeyedRecyclery<DataType, Initializer>::~KeyedRecyclery(void)
{
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
const KeyedRecyclery<DataType, Initializer>&
KeyedRecyclery<DataType, Initializer>::operator=(
	const KeyedRecyclery&	inSrc)
{
	mRecycleries.clear();
	return *this;
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
inline
DataType*
KeyedRecyclery<DataType, Initializer>::New(
	const Initializer&	inInitializer)
{
	CreateNewRecycleryIfNeeded(inInitializer);
	return mRecycleries[inInitializer].New(inInitializer);
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
inline
void
KeyedRecyclery<DataType, Initializer>::Dispose(
	const Initializer&	inInitializer,
	DataType*&			ioPtr)
{
	CreateNewRecycleryIfNeeded(inInitializer);
	mRecycleries[inInitializer].Dispose(ioPtr);
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
void
KeyedRecyclery<DataType, Initializer>::CreateNewRecycleryIfNeeded(
	const Initializer&	inInitializer)
{
	typename RecycleryMap::iterator i = mRecycleries.find(inInitializer);
	if (i == mRecycleries.end())
		mRecycleries[inInitializer] = Recyclery<DataType, Initializer>(inInitializer);
}


#endif // KeyedRecyclery_h__
