/*
 *  Model.cpp
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Model.h"
#include "PLEXApp.h"

#pragma mark -
//---------------------------------------------------------------------------
Model::	Model(
		const DoubleMatrix& inRateMatrix,
		const int& inNumStates,
		const IntVector& inStateIndex,
		const CharVector& inStates,
		const int& inMinState)
{
	num = &Numerics::getGlobalNumerics();

	mRateMatrix = inRateMatrix;
	mNumStates = inNumStates;
	mStateIndex = inStateIndex;
	mStates = inStates;
	mMinState = inMinState;
}		

//---------------------------------------------------------------------------
Model::Model(
		const Model&	inModel)
{
	num = &Numerics::getGlobalNumerics();

	mRateMatrix = inModel.mRateMatrix;
	mNumStates = inModel.mNumStates;
	mStateIndex = inModel.mStateIndex;
	mStates = inModel.mStates;
	mMinState = inModel.mMinState;
}

//---------------------------------------------------------------------------
Model::~Model(void)
{
}
										
//---------------------------------------------------------------------------
const Model& Model::operator=(
				const Model&	inModel)
{
	Model temp(inModel);
	swap(temp);
	return *this;
}
											
//---------------------------------------------------------------------------
void Model::swap(
			Model&	inModel)
{
	mRateMatrix.swap(inModel.mRateMatrix);
	std::swap(mNumStates, inModel.mNumStates);
	std::swap(mStateIndex, inModel.mStateIndex);
	std::swap(mStates, inModel.mStates);
	std::swap(mMinState, inModel.mMinState);
}

//---------------------------------------------------------------------------
void Model::InitializeParameter(
			const IntMatrix& m_ChangeMatrix)
{
}

//---------------------------------------------------------------------------
void Model::UpdateParameter(
			const int& m_nMethod)
{
}

//---------------------------------------------------------------------------
void Model::SetUpdateWindowsize(
			const double& inWindowsize)
{
}

//---------------------------------------------------------------------------
CharVector& Model::GetStates(void)
{
	return(mStates);
}
								
//---------------------------------------------------------------------------
int& Model::GetMinState(void)
{
	return(mMinState);
}
			
//---------------------------------------------------------------------------
IntVector& Model::GetStateIndex(void)
{
	return(mStateIndex);
}

//---------------------------------------------------------------------------
int& Model::GetNumStates(void)
{
	return(mNumStates);
}

//---------------------------------------------------------------------------
DoubleMatrix& Model::GetRateMatrix(void)
{
	return(mRateMatrix);
}

//---------------------------------------------------------------------------
const CharVector& Model::GetStates(void) const
{
	return(mStates);
}
								
//---------------------------------------------------------------------------
const int& Model::GetMinState(void) const
{
	return(mMinState);
}
			
//---------------------------------------------------------------------------
const IntVector& Model::GetStateIndex(void) const
{
	return(mStateIndex);
}

//---------------------------------------------------------------------------
const int& Model::GetNumStates(void) const
{
	return(mNumStates);
}

//---------------------------------------------------------------------------
const DoubleMatrix& Model::GetRateMatrix(void) const
{
	return(mRateMatrix);
}

//---------------------------------------------------------------------------
void Model::SetStates(
			const CharVector& inStates)
{
	mStates = inStates;
}
								
//---------------------------------------------------------------------------
void Model::SetMinState(
			const int& inMinState)
{
	mMinState = inMinState;
}
			
//---------------------------------------------------------------------------
void Model::SetStateIndex(
			const IntVector& inStateIndex)
{
	mStateIndex = inStateIndex;
}
			
//---------------------------------------------------------------------------
void Model::SetNumStates(
			const int& inNumStates)
{
	mNumStates = inNumStates;
}

//---------------------------------------------------------------------------
void Model::SetRateMatrix(
			const DoubleMatrix& inRateMatrix)
{
	mRateMatrix = inRateMatrix;
}

//---------------------------------------------------------------------------
void Model::PrintRateMatrix(
			ostream&		outStream, ObservedData& theData )
{
	PLEXApp* m_oPLEXApp = new(PLEXApp);

	m_oPLEXApp->PrintMatrix(outStream, mRateMatrix, theData);
	
	delete(m_oPLEXApp);
}

//---------------------------------------------------------------------------
void Model::PrintRateMatrixLine(
			ostream&		outStream)
{
	PLEXApp* m_oPLEXApp = new(PLEXApp);

	m_oPLEXApp->PrintMatrixLine(outStream, mRateMatrix);
	
	delete(m_oPLEXApp);
}

