/*
 *  Model.h
 /*
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



/*
 *
 *  this class is the general model for a single model
 *  it was only used when we simulate the mutation process in PLEX
 *  might be able to delete this one when we update the simulation process
 *
 */
 
#ifndef Model_h__
#define Model_h__

#include <algorithm>
#include <iostream>

#include "Numerics.h"
#include "CLAMPTypes.h"

class ObservedData;

using std::ostream;

//---------------------------------------------------------------------------
class Model
{
	public:
							Model(
								const DoubleMatrix& inRateMatrix = DoubleMatrix(),
								const int& inNumStates = 0,
								const IntVector& inStateIndex = IntVector(),
								const CharVector& inStates = CharVector(),
								const int& inMinState = 0);

							Model(
								const Model&	inModel);

			virtual				~Model(void);
										
			const Model&			operator=(
								const Model&	inModel);
											
			const CharVector&		GetStates(void) const;
								
			const int& 			GetMinState(void) const;
			
			const IntVector&		GetStateIndex(void) const;
			
			const int&			GetNumStates(void) const;

			const DoubleMatrix&		GetRateMatrix(void) const;
			
			CharVector&			GetStates(void);
								
			int& 				GetMinState(void);
			
			IntVector&			GetStateIndex(void);
			
			int&				GetNumStates(void);

			DoubleMatrix&			GetRateMatrix(void);
			
			void				SetStates(
								const CharVector& inStates);
								
			void				SetMinState(
								const int& inMinState);
			
			void				SetStateIndex(
								const IntVector& inStateIndex);
			
			void				SetNumStates(
								const int& inNumStates);
								
			void				swap(
								Model&	inModel);
								
			void 				PrintRateMatrix(
								ostream& outStream, ObservedData& theData );
								
			void				PrintRateMatrixLine(
								ostream& outStream);
								
			virtual void 			InitializeParameter(
								const IntMatrix& m_ChangeMatrix);

			virtual void 			UpdateParameter(
								const int& m_nMethod);
		
			virtual void 			SetUpdateWindowsize(
								const double& inWindowsize);

	protected:
			void				SetRateMatrix(
								const DoubleMatrix& inRateMatrix);

			DoubleMatrix			mRateMatrix;
			int				mNumStates;
			IntVector			mStateIndex;
			CharVector			mStates;
			int				mMinState;

			Numerics *num;			// [JdK] - an interface to accelerated numeric functions

	private:
};

#endif // Model_h__
