/*
 *  NewickTreeParser.h
 *  Originally created by Chip Jarred for David Pollock's CLAMP/PLEX program
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 *
 *  this class includes all the functions to parse the tree in newick format
 *
 */

#ifndef NewickTreeParser_h__
#define NewickTreeParser_h__

#include "TreeParser.h"

#include "Tree.h"

#include <deque>

//---------------------------------------------------------------------------

class NewickTreeParser : public TreeParser
{
	public:
		typedef	 TreeParser::node_type			node_type;
		typedef  TreeParser::tree_type			tree_type;
		typedef  TreeParser::node_type::node_list	node_list;
		
										NewickTreeParser(void);
		virtual							~NewickTreeParser(void);
		
		virtual	bool					Read(
											std::istream&		inTreeStream,
											tree_type&			outTree);
		virtual	bool					Write(
											std::ostream&		outTreeStream,
											const tree_type&	inTree);

	private:
		friend class UTNewickTreeParser;
		
				bool					ReadNode(
											std::istream&		inTreeStream,
											node_type*&			outNode);
				bool					ReadNodeList(
											std::istream&		inTreeStream,
											bool				inLimitToOne,
											node_list&			outNodes);
											
		static	const char*				cDelimiters;
		static	const char*				cQuotes;
		static	const char*				cTreeTerminator;
		static	const char*				cNodeSeparator;
		static	const char*				cOpenParentNodeDelimiter;
		static	const char*				cCloseParentNodeDelimiter;
		static	const char*				cWhitespaceChars;
		
		static	const char*				cTerminatorNotFound;
		static	const char*				cMissingCommaErr;
		static	const char*				cMultiNodesAtTopErr;
};


#endif // NewickTreeParser_h__
