/*
 *  NodeDistanceMaker.cpp
 *  Originally created by Chip Jarred for David Pollock's CLAMP/PLEX program
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 *  this class include functions to make a node distance and node name for building a tree
 *  
 */
 
#include "NodeDistanceMaker.h"

#include "TreeNode.h"

#include <cstdlib>
#include <cstring>

using std::atof;
using std::string;
using std::strncpy;
using std::max;

//---------------------------------------------------------------------------
unsigned int	NodeDistanceMaker::sNumNodes = 1;

const string	NodeDistanceMaker::cInitName;

#pragma mark -
//---------------------------------------------------------------------------
NodeDistanceMaker::~NodeDistanceMaker(void)
{
}

//---------------------------------------------------------------------------
NodeDistanceMaker::node_type*
NodeDistanceMaker::Make(
	const string&	inNodeDescription,
	node_type*		inNodeToSetFromDescription) const
{
	// if inNodeToSetFromDescription not NULL then it's id must be 0
	assert(inNodeToSetFromDescription == 0 || inNodeToSetFromDescription->GetData().id == 0);
	assert(inNodeToSetFromDescription == 0 || inNodeToSetFromDescription->GetData().name[0] == 0);
	
	double				distance		= GetInitDistance();
	string				name			= GetInitName();
	string::size_type	separatorLoc	= inNodeDescription.find(':');

	if (separatorLoc != string::npos) {
		name = inNodeDescription.substr(0, separatorLoc);
		distance = atof(inNodeDescription.substr(separatorLoc + 1).c_str());
	} else {
        cerr << endl << "   ERROR: Tree does not appear to have branchlengths.  Need starting branchlengths in treefile." << endl;
        exit(-1);
        
        name = inNodeDescription;
        distance = 0;
    }
	
	TreeNode* node = inNodeToSetFromDescription ? inNodeToSetFromDescription : new TreeNode();

	if (node) {
		string::size_type	nameLen = max(name.size(), static_cast<string::size_type>(MAXNAME));
		strncpy(node->GetData().name, name.c_str(), nameLen);
       
/*        if (distance == 0)
			node->GetData().distance = cSmallDistance;  // branch could not be zero, set a very small length
		else
			node->GetData().distance = distance;
*/
		
		node->GetData().distance = distance;

		if (distance <= 0)
			node->GetData().distance = cSmallDistance;  // branch could not be zero, set a very small length
        
		node->GetData().id = sNumNodes++;
	} else {
        cout << "   ERROR: FAILED TO CREATE NODE DURING TREE PARSE." << endl;
        exit(-1);
    }
	
	return node;
}

//---------------------------------------------------------------------------
inline
double
NodeDistanceMaker::GetInitDistance(void) const
{
	return cInitDistance;
}

//---------------------------------------------------------------------------
inline
const string&
NodeDistanceMaker::GetInitName(void) const
{
	return cInitName;
}
