//
/*  ObservedData.h
 /*
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef ObservedData_h__
#define ObservedData_h__

#include <string>
#include <map>
#include <vector>
#include "Matrix.h"
#include <iostream>
#include <fstream>
#include <sstream>
#include "Options.h"
#include "CLAMPTypes.h"

extern Options options;
using namespace std;

class ObservedData {
    private:
    // primary stuff
    int numSites;
    int numTaxa;
    int tupleSize;
    int numStates;
    
    // PRIMARY
    vector<string>          taxonNames;               // The taxon names
    vector< vector<int> >   theStates;                // THE SEQUENCE DATA (by taxon)
    map <string, int>       stateIndex;               // return the number given the character state
    map <int, string>       reverseStateIndex;        // return the character state given the number
    vector<bool>            anyGapPresent;            // for each site, is there a gap anywhere
    
    map <int, int>          realDatasetColumn;        // given an encoded column, return the real column in the alignment
    map <int, int>          reverseRealDatasetColumn; // (other way around)
    
    // SECONDARY (internals)
    bool            _gapsAsCharacters;                 // include gap states or not?
    int             _currentState;                     // a counter for import
    map< int, bool > _excludedPositions;               // which positions to exclude?
    bool            _hasGap( string );                 // check if character is a gap character
    bool            _getStatesFromData;                // create a new state index or not
    bool            _overlappingTuples;                // do we let tuples overlap?
    void            _removeAllGapColumns(void);        // prune theStates and set realDataColumn data
    
    public:

    DoubleVector            empiricalFreqs;
    
    ObservedData();
    void usePresetStateIndex(int);
    void readData( string fileName );
    void encodeSequence( string, vector<int> &);

    void setNumStates(int);
    void setTupleSize(int);
    
    void resetAlignmentLength( int val );
    int  getAlignmentLength(void);
    int  getNumSequences(void);
    int  getNumStates(void);

	string getState(int);
    bool isAnyGapPresent(int);
    
    string& getTaxonName_ref(int);
    std::vector<int>& getSequence_ref(int);

};

#endif
