/*
 *  Options.cpp
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Options.h"
#include "macrodefine.h"
#include <stdlib.h>

Options::Options() {
    // Computation
    randseed = 12345;
    numThreads = 1;
    m_nNumModels = 1;

    // Various defaults
    m_nMethod = 6;   // use B1u (other methods being deprectated)
    task = 0; 	      // default task (only 1 implemented now)
    Debugging = 1;    // ?
    m_bLoudness = 1;  // ?
    m_bSite = 1;      // obsolete
    m_bFixedFreq = 1; // obsolete

    // Data handling
	gapsAsMissing = 0; // by default, throw out gaps (or treat as extra state, depending on other options)
    
	// Global variables
	__ASRVmode = 1; // default to scaling the rate matrix for modelling rate variation
	__globalLikelihoodTemp = 1.0;
	__overrideSubResampling = 0;
    __overrideSiteResamp = 0;
    
    // Tree sampling
    sampleTrees = 0;
    treeSampleFreq = 0.001;
	numGensToReallocateProposals = 500;
    
    // Branch-length updaters
	updatebls = 0;
	brlenupdatefreq = 0.01;
	
	// Branch-length discretization
	maxBL = 0.08;
	numDiscreteBLs = 1;
	discreteBLs.push_back(0.005);
	HwangGreen = 0;
	
    // Model parameters
	m_nDatatype = datatype = 2;
	scaleByFrequencies = 0; //??
	scaleRateMatrix = 0;
	reversibleModel = 0;

    // Rate variation among sites (ASRV)
	allowASRV = 0;
	gammaShapeASRV = 0.19414;
	updateASRVparms = 0;
	numCatASRV = 1;

    // Discrete rate variation models (RAR)
	doRAR = 0;
	numCat = 5;
	fixedRates = -1;
	freeFreqs = 0;
	
    // Dependent rate assingment mixtures
	DRA = 0;
	DRAprior = 0.001;
    
    // Empirical stationary freqs?
    useEmpiricalStationary = 1;
    
	// MCMC parameters
	numGenerations = 100000;
	outputFrequency = 25;
	outputIncompleteLogL = 10000;
	capUniformizationRate = 0;
	capNumTreeNodes = false;
    ancStateUpdateFreq = 0.998;
	forceFullMarginalUpdate = 0;
	countExpectedSubstitutions = 0;
    freqAuxVirtualSubResampling = 0.2; // If mixing is poor, set high (1.0); if slow on large tree, set lower (0.8?)

    // Slice sampling defaults
    _slice_numGenerationsPerSample = 3;
    _slice_maximumWindowExpansions = 100;
    _slice_globalWindowMultiple    = 1.0;
    
    // Coevolution options
	coevol = 0;
	coevol_doPosteriorPredictiveNull = 0;
    coevol_numPPsitePairs = 10;
    coevol_thresholdForPP = 0.15;
	coevol_minNumSubstitutions = 1;
    coevol_outputDwell = 0;
    coevol_outputJointDwell = 0;
    coevol_DataReplacementSimulation = 0;
    coevol_DataReplacement_uniformASRVassign = 0;
    coevol_DataReplacement_fixSiteAssignments = 0;
    coevol_DataReplacement_differentNumSites = 0;
    
    // File output
	suppressFileOutput = 0;
	outputSiteRates = 0;
    
    // File output: substitution history sampling
    outputSubs = 1;
    outputSubFrequency = 1000;
    outputSubType = 1;
    

    // LEGACY STUFF
    m_strTreefile = (char*) malloc(sizeof(char) * MAXNAME);
    m_strSeqfile = (char*) malloc(sizeof(char) * MAXNAME);
    m_strPositionfile = (char*) malloc(sizeof(char) * MAXNAME);
    m_strLengthfile = (char*) malloc(sizeof(char) * MAXNAME);
    m_strOLfile = (char*) malloc(sizeof(char) * MAXNAME);
    m_strTreeOutfile = (char*) malloc(sizeof(char) * MAXNAME);
    m_strSubOutfile = (char*) malloc(sizeof(char) * MAXNAME);
    m_strSeqOutfile = (char*) malloc(sizeof(char) * MAXNAME);
    m_strCountOutfile = (char*) malloc(sizeof(char) * MAXNAME);
    m_strLikelihoodOutfile = (char*) malloc(sizeof(char) * MAXNAME);
    m_strMatrixOutfile = (char*) malloc(sizeof(char) * MAXNAME);

}
