/*
 *  Options.h
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef Options_h__
#define Options_h__

#include <vector>
#include <string>

class Options {
public:
    Options();

    // Some globals
    double __largestRate;  // a terrible global variable! oh no!
    int    __ASRVmode;	   // another one: which mode are we using for ASRV: 1=rates; 0=bls (only applies to B1u)
    double __globalLikelihoodTemp;
    int    __overrideSubResampling; // temporary global for diagnostics
    int    __overrideSiteResamp; // global - temp to block reassignment of rates
    int __currentGen;
    int __initialNumBranches;
    double __currentRateSum;

    // Initialize stationary distribution using empirical frequencies?
    int useEmpiricalStationary;
    
    // Random number generation
    int randseed;
    
    // Operating mode for prpogram
    int task;
    
    // For missing data handling
    int gapsAsMissing;
    
    // Branch length discretization and/or subdivision...
    double maxBL;
    double maxExpectedSubs;
    
    int numDiscreteBLs;
    std::vector<double> discreteBLs;
    int HwangGreen;

    // Tree sampling
    std::vector<std::string> treeFiles;
    int sampleTrees;
    double treeSampleFreq; // how often to run the tree sampler
    int numGensToReallocateProposals;
    
    // Ancestral state sampling...
    double ancStateUpdateFreq;
    int forceFullMarginalUpdate;
    int countExpectedSubstitutions;
    double freqAuxVirtualSubResampling; // how often to resample virtual substitutions in addition to standard to aide mixing
    
    // Calculate and output the incomplete logL (and sample states) every x gens
    int outputIncompleteLogL;
    
    // SMP using OpenMP
    int numThreads;		// number of threads to use (OpenMP)

    // Model parameters
    int datatype;
    int scaleByFrequencies;
    int reversibleModel;
    int scaleRateMatrix;

    // Discrete rate variation models (RAR)...
    int doRAR;
    int numCat;
    int fixedRates;
    int freeFreqs;

    // Rate variation across sites (ASRV)
    double gammaShapeASRV;
    int allowASRV;
    int updateASRVparms;
    int numCatASRV;
   
    // Dependent rate assignment mixture parameters
    int DRA;
    double DRAprior;
    
    // MCMC parameters
    int numGenerations;
    int updatebls;
    double brlenupdatefreq;
    int capUniformizationRate;
    bool capNumTreeNodes;
    
    // Slice sampling parameters
    int     _slice_numGenerationsPerSample;
    int     _slice_maximumWindowExpansions;
    double  _slice_globalWindowMultiple;
    
    // Coevolution options
    int coevol;
    int coevol_doPosteriorPredictiveNull;
    int coevol_numPPsitePairs;
    double coevol_thresholdForPP;
    int coevol_minNumSubstitutions;
    int coevol_outputDwell;
    int coevol_outputJointDwell;
    int coevol_DataReplacementSimulation; // this will simulate new data at generation 6k
    int coevol_DataReplacement_uniformASRVassign; // assign site rates uniformly from distribution
    int coevol_DataReplacement_fixSiteAssignments;
    
    int coevol_DataReplacement_differentNumSites;
    int coevol_DataReplacement_newNumSites;
    
 	// File output control
    int suppressFileOutput;
    int outputSiteRates;
    int outputFrequency;
    
    // Output of substitution mappings:
    int outputSubs;
    int outputSubFrequency;
    int outputSubType;
    
    // LEGACY OPTIONS GO HERE
    int m_nDatatype, m_nGenerations, m_nBurnin, m_nSeqfreq, m_nMatfreq, m_nProbfreq, m_nCountfreq, m_nMethod, m_nWindowThreshold, m_nTotalWindow, m_nMaxWindow, m_nSimulationGeneration,  m_nNumModels;
    bool Debugging, m_bLoudness, outputTrees, outputSequences, m_bMatflag, m_bCountflag, m_bLikeflag, m_bUpdate, m_bFixedFreq, m_bSite;
    double m_fUpdatewindow;
    
    int m_nNumBranches;
    char* m_strTreefile;
    char* m_strSeqfile;
    char* m_strPositionfile;
    char* m_strLengthfile;
    char* m_strOLfile;
    char* m_strTreeOutfile; // Validated
    char* m_strSubOutfile; // Validated
    char* m_strSeqOutfile;
    char* m_strCountOutfile;
    char* m_strLikelihoodOutfile;
    char* m_strMatrixOutfile;
    
};

#endif