//
//  PartialLogLikelihood.h Created for PLEXlite by A.P. Jason de Koning on 1/23/12.
/*
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PLEXlite_PartialLogLikelihood_h
#define PLEXlite_PartialLogLikelihood_h

class Numerics;
class ModelPropertiesForUpdate;

#include "CLAMPTypes.h"

class PartialLogLikelihood {
public:
    PartialLogLikelihood();
    PartialLogLikelihood( int xInit, int yInit, double muInit, double C1init, double CXYinit, DoubleMatrix& rateMatrix, DoubleMatrix& alphaMatrix, DoubleVector& stateFreqs  );
    
    // Want the logL to be overridden by every sub-class
    virtual double logL( double value ) = 0;
    
    void setParms( ModelPropertiesForUpdate& modelProperties );
    
    void setBounds( double max, double min );
    void setModel( int model );
    
protected:
    Numerics *num;
    DoubleMatrix *rates;
    DoubleMatrix* alpha;
    
    DoubleVector* siteRates;
    DoubleVector* stateFreqs;

    DoubleMatrix* CXYs;
    DoubleMatrix* C1s;
    
    double min, max;
    double mu;
    int x,y;
    int theModel;
    double C1, CXY;
};

// Individual rate parameters
class PartialLogLikelihood_individualRateParm : public PartialLogLikelihood {
public:
    PartialLogLikelihood_individualRateParm();
    PartialLogLikelihood_individualRateParm( int xInit, int yInit, double muInit, double C1init, double CXYinit, DoubleMatrix& rateMatrix );
    
    double logL( double value );
};

// Reversible model parameters
class PartialLogLikelihood_reversibleRateParm : public PartialLogLikelihood {
public:
    PartialLogLikelihood_reversibleRateParm();
    
    PartialLogLikelihood_reversibleRateParm( int xInit, int yInit, double muInit, double C1init, double CXYinit, double C1init_a, double CXYinit_a, DoubleMatrix& rateMatrix, DoubleMatrix& alphaMatrix, DoubleVector& stateFreqs );

    double logL( double value );

protected:
    double C1a, CXYa;
};

#endif
