//
//  PriorDistributions.h Created for PLEXlite by A.P. Jason de Koning on 1/25/12.
/*
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PLEXlite_PriorDistributions_h
#define PLEXlite_PriorDistributions_h

class Numerics;

class PriorDistribution {
public:
    // Constructor must allocate parameters vector
    PriorDistribution();
    
    // Distribution parameters
    std::vector<double> hyperParameters;
    
    // Need a function for evaluating PDF
    virtual double evaluatePDF( double x, std::vector<double> &distParms ) = 0;
    virtual double evaluatePDF( double x ) = 0;
    
    // Automate log-PDF when it is easy to override
    virtual double evaluateLogPDF( double x );
    
    // Common function to set parameters
    void setParameters( std::vector<double> &parms );
    
protected:
    Numerics *num;
};

class UniformPrior : public PriorDistribution {
public:
    UniformPrior();

    double evaluatePDF( double x, std::vector<double> &distParms );
    double evaluatePDF( double x );
    double evaluateLogPDF( double x );
};

class BetaPrior : public PriorDistribution {
public:
    BetaPrior();
    BetaPrior( double a, double b);
    
    double evaluatePDF( double x, std::vector<double> &distParms );
    double evaluatePDF( double x );
    double evaluateLogPDF( double x );
};

class GammaPrior : public PriorDistribution {
public:
    GammaPrior();
    GammaPrior( double a, double b );
    double evaluatePDF( double x, std::vector<double> &distParms );
    double evaluatePDF( double x );
};

class ExpPrior : public PriorDistribution {
public:
    ExpPrior();
    ExpPrior( double lambda );
    
    double evaluatePDF( double x, std::vector<double> &distParms );
    double evaluatePDF( double x );
    double evaluateLogPDF( double x );
};

class GaussianPrior : public PriorDistribution {
public:
    GaussianPrior();
    GaussianPrior( double mu, double sigma );
    
    double evaluatePDF( double x, std::vector<double> &distParms );
    double evaluatePDF( double x );
};

#endif
