/*
 *  RandomGenerator.cpp
 *  Originally created by Chip Jarred for David Pollock's CLAMP/PLEX program
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "RandomGenerator.h"
#include <cmath>

#pragma mark -
//---------------------------------------------------------------------------
RandomGenerator::RandomGenerator(void)
	:	mW(cWDefault),
		mX(cXDefault),
		mY(cYDefault),
		mZ(cZDefault)
{
}

//---------------------------------------------------------------------------
double
RandomGenerator::Random(void)
{
	if (cFastRandomNumber)
	{
		mW *= 127773;
		return ldexp((double)mW, -32);
	}
	else
	{
		// U(0,1): AS 183: Appl. Stat. 31:188-190 
		// Wichmann BA & Hill ID.  1982.  An efficient and portable
		// pseudo-random number generator.  Appl. Stat. 31:188-190
		//
		//	x, y, z are any numbers in the range 1-30000.  Integer operation up
		//	to 30323 required.

		mX = 171 * (mX % 177) -  2 * (mX / 177);
		mY = 172 * (mY % 176) - 35 * (mY/ 176);
		mZ = 170 * (mZ % 178) - 63 * (mZ / 178);

		// if (mX<0)
		// 	mX+=30269;
		// if (mY<0)
		// 	mY+=30307;
		// if (mZ<0)
		// 	mZ+=30323;

		double r = mX / 30269.0 + mY / 30307.0 + mZ / 30323.0;
		return (r - static_cast<int>(r));
	}
}

//---------------------------------------------------------------------------
void
RandomGenerator::SetSeed(
	int	inSeed)
{
   mZ = 170 * (inSeed % 178) + 137;
   mW = inSeed;
}

//---------------------------------------------------------------------------
RandomGenerator&
RandomGenerator::GetGlobalGenerator(void)
{
	static RandomGenerator globalGenerator;
	return globalGenerator;
}

