/*
 *  Recyclery.h
 *  Originally created by Chip Jarred for David Pollock's CLAMP/PLEX program
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 *
 * recycle the memory cost for a data type, not used in current release
 *
 */

#ifndef Recyclery_h__
#define Recyclery_h__

#include <algorithm>
#include <stack>

template <typename DataType, typename Initializer>
class Recyclery
{
	public:
													Recyclery(void);
													Recyclery(
														const Initializer&	inInitializer);
													Recyclery(
														const Recyclery&	inSrc);
													~Recyclery(void);
													
				const Recyclery&					operator=(
															const Recyclery& inSrc);

				DataType*							New(void);
				DataType*							New(
														const Initializer&	inInitializer);
				void								Dispose(
														DataType*&	ioPtr);
									
				
	protected:
	private:
				std::stack<DataType*>				mRecycleBin;
				Initializer							mInitializer;
};

#pragma mark -
//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
Recyclery<DataType, Initializer>::Recyclery(void)
	:	mRecycleBin(),
		mInitializer()
{
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
Recyclery<DataType, Initializer>::Recyclery(
	const Recyclery&	inSrc)
	:	mRecycleBin(),
		mInitializer(inSrc.mInitializer)
{
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
Recyclery<DataType, Initializer>::Recyclery(
	const Initializer&	inInitializer)
	:	mRecycleBin(),
		mInitializer(inInitializer)
{
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
Recyclery<DataType, Initializer>::~Recyclery(void)
{
	while(mRecycleBin.size())
	{
		delete mRecycleBin.top();
		mRecycleBin.pop();
	}
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
const Recyclery<DataType, Initializer>&
Recyclery<DataType, Initializer>::operator=(
	const Recyclery&	inSrc)
{
	mInitializer = inSrc.mInitializer;
	
	while(mRecycleBin.size())
		mRecycleBin.pop();
		
	return *this;
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
DataType*
Recyclery<DataType, Initializer>::New(void)
{
	return New(mInitializer);
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
DataType*
Recyclery<DataType, Initializer>::New(
	const Initializer&	inInitializer)
{
	DataType*	ptr;
	if (mRecycleBin.size())
	{
		ptr = mRecycleBin.top();
		mRecycleBin.pop();
		ptr->Initialize(inInitializer);
	}
	else ptr = new DataType(inInitializer);
	return ptr;
}

//---------------------------------------------------------------------------
template <typename DataType, typename Initializer>
void
Recyclery<DataType, Initializer>::Dispose(
	DataType*&	ioPtr)
{
	mRecycleBin.push(ioPtr);
	ioPtr = 0;
}

#endif // Recyclery_h__
