//
/*  SimplifiedLogL.h Created by A.P. Jason de Koning on 1/23/12. ( PLEXlite)
 
PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
version 0.94 - beta version July, 2012

Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
jason.de.koning@gmail.com http://jasondk.org
wanjungu@gmail.com
David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com

This program was created and distributed by the
David Pollock laboratory, University of Colorado School of Medicine
copyright © 2012 David Pollock
All rights reserved.

http://www.evolutionarygenomics.com
Dept. Biochemistry & Molecular Genetics
12801 17th Ave., MS 8101
PO Box 6511
Aurora, CO 80045

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PLEXlite_SimplifiedLogL_h
#define PLEXlite_SimplifiedLogL_h

#include "CLAMPTypes.h"

// Interfaces to automate logL calculations
class Numerics;

class SimplifiedLogL {
public:
    SimplifiedLogL();
    SimplifiedLogL( int xInit, int yInit, double muInit, double C1init, double CXYinit, DoubleMatrix& rateMatrix );
    
    // Want the logL to be overridden by every sub-class
    virtual double logL( double value ) = 0;
    
    void setParms( int xInit, int yInit, double muInit, double C1init, double CXYinit, DoubleMatrix& rateMatrix );
    
protected:
    Numerics *num;
    
    DoubleMatrix rates;
    double mu;
    int x,y;
    double C1, CXY;
    
};

// Individual rate parameters

class SimplifiedLogL_individualRateParm : public SimplifiedLogL {
public:
    SimplifiedLogL_individualRateParm();
    SimplifiedLogL_individualRateParm( int xInit, int yInit, double muInit, double C1init, double CXYinit, DoubleMatrix& rateMatrix );
    
    double logL( double value );
};



#endif
