/*
 *  SiteCategory.h
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef SiteCategory__h__
#define SiteCategory__h__

#include "CLAMPTypes.h"

class SiteCategory {
	public:
		double		 gammaShapeParameter_ASRV;
		DoubleVector siteRateDistribution;
		DoubleVector scalingFactors;
	
		IntVector	 siteAssignments;	// for each site, which category is assigned?
		IntVector	 backupAssignments;	// for each site, which category is assigned?
		
	int setGammaDistribution(void);
	int setGammaDistribution(double shapeParm);

	// Constructor:
	SiteCategory(int numSites);

	// Member functions:
	int getBackupAssignment(int site) { return backupAssignments.mDataArray[site]; }
	int getAssignment(int site) { return siteAssignments.mDataArray[site]; }
	void setAssignment( int site, int state ) { siteAssignments[site] = state; }

	double getRate(int which) { return siteRateDistribution.mDataArray[which]; }
	double getShapeParm(void) { return gammaShapeParameter_ASRV; }
	void   setShapeParm(double val) { gammaShapeParameter_ASRV = val; }
	
	void backupSiteAssignments(void) { for (int i=0; i<siteAssignments.size(); i++) { backupAssignments[i] = siteAssignments[i]; } };
	void restoreSiteAssignments(void) { for (int i=0; i<siteAssignments.size(); i++) { siteAssignments[i] = backupAssignments[i]; } };
};

#endif