/*
 *  Tokenizer.h
 *  Originally created by Chip Jarred for David Pollock's CLAMP/PLEX
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.
 
 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef Tokenizer_h__
#define Tokenizer_h__

#include <iosfwd>
#include <memory>
#include <string>

class Tokenizer_Impl;

class Tokenizer
{
	public:
														Tokenizer(void);
														Tokenizer(
															const Tokenizer&	inSrc);
		virtual											~Tokenizer(void);
		
				const Tokenizer&						operator=(
															const Tokenizer&	inSrc);
															
				void									SetStream(
															std::istream&		inStream);
				bool									HasStream(void) const;
		
				bool									ReadToken(
															std::string&		outToken);
				void									PutBackToken(
															const std::string&	inToken);
															
				void									Reset(void);
				
				void									EatWhitespace(void);

				void									SetDelimiters(
															const std::string&	inDelimiters);
				const std::string&						GetDelimiters(void) const;
				bool									IsDelimiter(
															const std::string&	inToken) const;
				
				void									SetWhitespaceChars(
															const std::string&	inWhitespaceChars);
				const std::string&						GetWhitespaceChars(void) const;
				
				void									SetQuoteChars(
															const std::string&	inQuoteChars);
				const std::string&						GetQuoteChars(void) const;
				
				void									SkipWhitespace(
															bool				inSkipWhitespace = true);
				bool									SkippingWhitespace(void) const;
				
				void									RemoveQuotes(
															bool				inRemoveQuotes = true);
				bool									RemovingQuotes(void) const;

	protected:
	private:
				void									Swap(
															Tokenizer&		ioSwapee);
														
				std::auto_ptr<Tokenizer_Impl>	mImpl;
};

#endif // Tokenizer_h__
