/*
 *  TreeNodeData_.h
 *  Originally created by Wanjun Gu for the CLAMP precursor to PLEX
 *  Copyright (c) 2004 Dr. David Pollock. All rights reserved.

 PLEX - Phylogenetics, Likelihood, Evolution, and Complexity
 version 0.94 - beta version July, 2012
 
 Authors: A.P. Jason de Koning, Wanjun Gu and David Pollock
 jason.de.koning@gmail.com http://jasondk.org
 wanjungu@gmail.com
 David.Pollock@UCDenver.edu  http://www.evolutionarygenomics.com
 
 This program was created and distributed by the
 David Pollock laboratory, University of Colorado School of Medicine
 copyright © 2012 David Pollock
 All rights reserved.
 
 http://www.evolutionarygenomics.com
 Dept. Biochemistry & Molecular Genetics
 12801 17th Ave., MS 8101
 PO Box 6511
 Aurora, CO 80045
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 *  This class defines the data type of each tree node
 */

#ifndef TreeNodeData__h__
#define TreeNodeData__h__

//include "Sequence.h"
#include "SiteCategory.h"
#include "CLAMPTypes.h"
#include "macrodefine.h"

class TreeNodeData_ {
	public:
		char name[MAXNAME];		/* node name */
		double distance;		/* associated with up branch */
		
		int id; //,tips;	  	/* node identification; number of tips below */
		std::vector<int>  sequence;		/* sequence information */
        std::vector<bool> seq_missingMask;
    
		IntMatrix countchange;		/* for change of each states with up branch*/
		
		DoubleMatrix3D probmatrix3D;	/* the probability matrix of the up node to current node*/
		DoubleMatrix3D *minimalBLprobmatrix3D;	// this allows each mixture component to have a minimal (but non-zero) model for B1xM
		
		DoubleMatrix3D countModelchange3D;	/* the changes of sites assigned to different models */
		DoubleMatrix   augmentedCounts;
        DoubleMatrix   augmentedCounts_bl; // duplicate counts of virtual substitutions using bl scaling mode
    
		IntVector	 totalCount_actualSubs;

	// [don't need this]	DoubleMatrix expectedVirtualSubsByRateCat;		// for rate, state
	
		DoubleMatrix augmentedLogSum;
		DoubleMatrix3D countModelchange3Duni;	// partial change count path stuff
	
		DoubleMatrix augmentedLogSum_scaling;
		DoubleMatrix3D ChangeCount3D_scaling;

		DoubleVector tempTreeDepth;
		DoubleMatrix marginalDwellTimes;
		DoubleMatrix observedFreqs;
		DoubleMatrix observedJFreqs;
        DoubleMatrix jointDwellTimes;
	
		double C0, C1, C1v;
	
		DoubleMatrix Cxx; // this will hold counts of Cxx events for each model type
	
		double treeLength;
	
		Vector<DoubleMatrix> countModelchange_perSite;	/* the changes of sites assigned to different models */
        DoubleMatrix timeSpentSame_perSite;

		// Need these for ancestral state reconstructions...
		double *siteLikelihoodBelow, *siteLikelihoodAbove, *siteLikelihoodBelow_andAbove; //, *siteLikelihoodL, *siteLikelihoodR;
		int siteLallocatedFlag;
		Vector<int> seqBackup;
		
        std::vector<int> posteriorPredictiveSeq;	// so we can simulate a site from the posterior predictive distribution
		int branchLengthIndex;
		int modelIndex;
		
		int UVRallocated;
		double *U, *V, *Root; // these are to be used at the root only
		
		SiteCategory *siteCats;
	
			// int marked;	// temp for temporal
		
		int doScaling;
		DoubleVector scalingFactor;
		
	void allocateForSiteLikelihoods(int numSites, int numStates) { if (siteLallocatedFlag ==1) { return; } siteLallocatedFlag = 1; siteLikelihoodBelow = new double[numStates * numSites];  } 
		// siteLikelihoodBelow_andAbove = new double[numStates * numSites]; siteLikelihoodAbove =  new double[numStates * numSites];
		// siteLikelihoodL =  new double[numStates * numSites]; siteLikelihoodR =  new double[numStates * numSites]; }

	void deallocateSiteLikelihoods(void) { siteLallocatedFlag = 0; delete siteLikelihoodBelow; } 
		//, siteLikelihoodAbove, siteLikelihoodBelow_andAbove; } //, siteLikelihoodL, siteLikelihoodR; }
		
		TreeNodeData_(void);

		TreeNodeData_(
					const TreeNodeData_& inSrc);

		const TreeNodeData_&	operator=(
					const TreeNodeData_& inSrc);

	private:
} ;

#endif // TreeNodeData__h__
